//---------------------------------------------------------------------------
#include <hpgcc49.h>

#include "Board.h"
//---------------------------------------------------------------------------

#define INT_MAX 0x7fffffff

Board::~Board()
{
    delete next;
    next = NULL;
}

// Assignment operator.
// Copy the squares and the log function pointer.
// Don't copy any "next" ptr because the src board owns the next board.
// If you want to change this, then you must make a copy of the next board
// and point "next" to the copy.  Otherwise memory corruption will result.
Board &
Board::operator=(const Board &src)
{
    int i,j;
    for (i=0; i < 9; ++i) {
        for (j=0; j < 9; ++j) {
            squares[i][j] = src.squares[i][j];
        }
    }
    next = NULL;
    return *this;
}


// Equality operator.  Boards are equal if the squares are equal
bool
Board::operator ==(const Board &b) const
{
    int i,j;
    for (i=0; i < 9; ++i) {
        for (j=0; j < 9; ++j) {
            if (!(squares[i][j] == b.squares[i][j])) {
                return false;
            }
        }
    }
    return true;
}

// Clear the board back to an empty state
void
Board::clear()
{
    int i,j;
    for (i=0; i < 9; ++i) {
        for (j=0; j < 9; ++j) {
            squares[i][j].clear();
        }
    }
}

// forEachGroup lets you call a function that processes each group in the
// puzzle (each row, column, and region).  The indices of the squares are
// passed to FUNC() in SQUARES. FUNC also gets passed the PARAMS
// that you pass in.  FUNC should return NULL unless forEachGroup should be
// aborted, in which case forEachGroup will return the first non-NULL
// return value from FUNC.
void *
Board::forEachGroup(ForEachGroupCallback func, void *params)
{
    int i,j, idx;
    void *result;
    int startRow, startCol;
    Square *group[9];         // The squares array to call func with

    // Note: this could be sped up by pre-computing the arrays that you
    // pass.

    // Do the rows & columns
    for (i=0; i < 9; ++i) {
        for (j=idx=0; j < 9; ++j) {
            group[idx++] = &squares[i][j];
        }
        result = (this->*func)(group, params);
        if (result) return result;

        for (j=idx=0; j < 9; ++j) {
            group[idx++] = &squares[j][i];
        }
        result = (this->*func)(group, params);
        if (result) return result;
    }

    // Do the regions
    for (startRow=0; startRow < 9; startRow += 3) {
        for (startCol=0; startCol<9; startCol += 3) {
            int idx = 0;
            for (i=startRow; i < startRow+3; ++i) {
                for (j = startCol; j<startCol+3; ++j) {
                    group[idx++] = &squares[i][j];
                }
            }
            result = (this->*func)(group, params);
            if (result) return result;
        }
    }
    return NULL;
}

// Return true if the board is valid, otherwise false.  A valid board
// contains no duplicate values in any row, column or region.
bool
Board::isValid()
{
    // Validate each row, column, or region.
    void *p = forEachGroup(&Board::validateGroup, NULL);
    return p == NULL;
}


// forEachGroup() callback to validate a single group.
void *
Board::validateGroup(Square *group[], void *)
{
    Bitmap taken;    // bitmap of the values used so far in this group
    int idx;
    int val;

    // Go through the group, remembering the values that you've seen so far
    // in "taken".  At each square, check if its value is already taken and then
    // add the value to those taken.
    for (idx = 0; idx<9; ++idx) {
        val = group[idx]->knownValue;
        if (val) {
            if (taken.check(val)) {
                return (void *)1;   // group not valid;
            }
            taken += val;
        }
    }
    return NULL;
}

// Look at the possibilities in the squares and set known values
// for any unknown values that have just one possibility;
void
Board::setKnownValues()
{
    int i,j;
    for (i=0; i < 9; ++i) {
        for (j=0; j < 9; ++j) {
            if (squares[i][j].knownValue == 0) {
                squares[i][j].setKnownValue();
            }
        }
    }
}

// Given the x&y coordinates for a square with a known value, remove
// that value from the possibilities of the row, column, and region that
// the square belongs to
void
Board::reduceSingle(int x, int y)
{
    int i,j;
    int val;    // value of square[x][y]
    int startx, starty;

    val = squares[x][y].knownValue;
    if (val == 0) {
        return; // It isn't a known value yet!
    }

    for (i=0; i<9; ++i) {
        // do the row and the column
        squares[i][y].possibilities -= val;
        squares[x][i].possibilities -= val;
    }

    // do the region
    startx = x/3 * 3;   // starting x index
    starty = y/3 * 3;   // starting y index
    for (i=startx; i < startx+3; ++i) {
        for (j=starty; j < starty+3; ++j) {
            squares[i][j].possibilities -= val;
        }
    }
}



    // Thow away and recompute the possible values of each square from the known values
void
Board::calcPossibilities()
{
    int i,j;
    for (i=0; i < 9; ++i) {
        for (j=0; j < 9; ++j) {
            // clear and reset the square
            int val = squares[i][j].knownValue;
            squares[i][j].clear();
            if (val) {
                squares[i][j].setVal(val);
            }
        }
    }
    // now set the possibilities
    reduce();
}

// Given a pointer to a square, return its coordinates in the board
void
Board::coordinates(int &i, int &j, const Square &s) const
{
    int idx = &s - &squares[0][0];
    i = idx / 9;
    j = idx % 9;
}



// return true of the board is solved.
bool
Board::isSolved()
{
    int i,j;
    setKnownValues();
    for (i=0; i < 9; ++i) {
        for (j=0; j < 9; ++j) {
            if (!squares[i][j].knownValue) {
                return false;
            }
        }
    }
    // If you get here then all squares have a known value
    return true;
}

#if 0
//---------------------------------------------------------------------------
// I/O routines.
// The format is one line per row.  Lines beginning with '#' are ignored.
// Otherwise the line is at least 9 charcters.  characters 1-9 are the value and
// anything else means a space.  Characters beyond 9 are ignored
// Example:
// # Here is a puzzle
// 123...789
// 789   123
// 456XXX312
//
// etc
//
// To suppose the sdm format, this will also read a 91-character line
bool
Board::read(FILE *fp)
{
    char buf[1000];
    int len;            // strlen(buf)
    int i=0, j=0;
    clear();
    if (feof(fp)) {
        return false;
    }
    while (j<9 && fgets(buf, sizeof(buf), fp)) {
        if (*buf == '#') continue;

        // remove trailing newline
        buf[sizeof(buf)-1] = 0;
        len = strlen(buf);
        if (buf[len-1] == '\n') {
            buf[--len] = 0;
        }
        if (len < 9) {
            return false;
        }

        if (len == 9*9) {
            const char *cp = buf;
            // sdm format - one puzzle per line
            for (j=0; j<9; ++j) {
                for (i=0; i<9; ++i) {
                    char ch = *cp++;
                    if (ch >= '1' && ch <= '9') {
                        squares[j][i].setVal(ch - '0');
                    }
                }
            }
            break;
        }

        for (i=0; i<9; ++i) {
            char ch = buf[i];
            if (ch >= '1' && ch <= '9') {
                squares[j][i].setVal(ch - '0');
            }
        }
        ++j;
    }
    return true;
}


bool
Board::write(FILE *fp)
{
    for (int i=0; i<9; ++i) {
        for (int j=0; j<9; ++j) {
            int val = squares[i][j].knownValue;
            if (val) {
                fputc(val+'0', fp);
            } else {
                fputc('.', fp);
            }
        }
        fputc('\n', fp);
    }
    return true;
}
#endif

//---------------------------------------------------------------------------
// Reduce algorithm.  If you can rule out 8 possibilities for a given square,
// then the 9th possibility must be the value in that square.  Actually, the
// reduce() method here just sets the possibilities in each square.  You then need
// to call setKnownValues() to check whether any squares have just one possibilitity
// as a result and get the known values set.
void
Board::reduce()
{
    forEachGroup(&Board::reduceCB, NULL);
}

void *Board::reduceCB(Square *group[], void *)
{
    int idx;
    Bitmap known;

    // gather a bitmap of all the known values in the group
    for (idx=0; idx < 9; ++idx) {
        if (group[idx]->knownValue) {
            known += group[idx]->knownValue;
        }
    }

    // clear the known values from the possibilities in the unknown
    // squares.
    for (idx=0; idx < 9; ++idx) {
        if (!group[idx]->knownValue) {
            group[idx]->possibilities -= known;
        }
    }
    return NULL;
}


// Set the square to the known value val and then remove val from
// the possibilities of all squares in the same row, column and region.
void
Board::setAndReduce(Square &s, int val)
{
    int i,j;   // row and column for this square
    s.setVal(val);
    coordinates(i,j, s);
    reduceSingle(i,j);
}


//---------------------------------------------------------------------------
// Scan algorithm.
// forEach callback for scanning algorithm.  See if the value v can go into just
// one of the squares in the group.
#if 1
void *
Board::scan(Square *group[], void *v)
{
    int count = 0;  // number of places where val can appear
    int r = -1;
    int c = -1;     // last column where val can appear.
    int idx;
    int val = (int) v;

    for (idx=0; idx < 9; ++idx) {
        if (group[idx]->knownValue == val) {
            return NULL;     // val is already taken
        }
        if(group[idx]->isPossible(val)) {
            ++count;
            coordinates(r, c, *group[idx]);
        }
    }
    if (count == 1) {
        // There was only one possible spot for this value.  Use it.
        // Note that you must reduce after setting the value because you
        // rely on the possibilities bitmap.
        setAndReduce(squares[r][c], val);
    }
    return NULL;
}
#else
void *
Board::scan(Squares *group[], void *)
{
    // This is a faster but more complicated version
    // of the scan algorithm.

    // If a bit is set in this bitmap, then the
    // corresponding number is a possibilitiy in AT
    // LEAST one square
    Bitmap atLeastOnce;

    // If a bit is set in this bitmap, then the
    // corresponding number is a possibility in MORE
    // THAN ONE square
    Bitmap moreThanOnce;

    for (int idx = 0; idx < 9; ++idx) {
        if (group[idx]->knownValue) {
            continue;
        }
        moreThanOnce += atLeastOnce & group[idx]->possibilities;
        atLeastOnce += group[idx]->possibilities;
    }

    Bitmap exactlyOnce = atLeastOnce & ~moreThanOnce;

    if (!exactlyOnce.isEmpty()) {
        // Some values are possible in only one spot.
        // Find them and set them
        for (int idx = 0; idx < 9; ++idx) {
            if (group[idx]->knownValue) {
                continue;
            }
            Bitmap tmp(group[idx]->possibilities & exactlyOnce);
            if (!tmp.isEmpty()) {
                group[idx]->possibilities = tmp; // so setKnownValue will work
                group[idx]->setKnownValue();
            }
        }
    }
    return NULL;
}
#endif


//---------------------------------------------------------------------------
// Pairs algorithm.

// ForEachGroup callback to check for "naked pairs."  For example, if there are
// two squares in the group whose only possibilities are 4 and 5, then one
// of them MUST be 4 and the other MUST be 5.  Therefore, no OTHER squares
// in the group can be either 4 or 5.
// Note that the board must be reduced before calling this.
void *
Board::pairCheck(Square *group[], void *)
{
    int idx1;
    int idx2;

    // Go through the first 8 squares in the group...
    for (idx1 = 0; idx1<8; ++idx1) {
        if (group[idx1]->knownValue) continue;

        // ... and consider it with each of the squares that comes after it
        // in the array.  For example, if idx1 is square 3, then we consider it
        // in combination with squares 4,5,6,7,8,9
        for (idx2 = idx1+1; idx2<9; ++idx2) {
            if (group[idx2]->knownValue) continue;

            if (group[idx1]->possibilities == group[idx2]->possibilities) {
                Bitmap possibilities = group[idx1]->possibilities;
                int bits = possibilities.count();
                if (bits == 2) {
                    // These two squares each have 2 possibilities and the
                    // possibilities are the same.  Remove the possibilities
                    // from everyone else in the group
                    for (int j=0; j < 9; ++j) {
                        // Don't remove it from either of the two squares that
                        // make up the pair!
                        if (j == idx1 || j == idx2) continue;
                        group[j]->possibilities -= possibilities;
                    }
                }
            }
        }
    }
    return NULL;
}

//---------------------------------------------------------------------------
// Naked Tuples algorithm.

// This is like pairCheck, but it does larger tuples also.  For example,
// 3 squares with the same 3 possibilities.
void *
Board::tupleCheck(Square *group[],void *)
{
    int idx1;
    int idx2;
    int count;

    // For each square with unknown value...
    for (idx1 = 0; idx1<8; ++idx1) {
        if (group[idx1]->knownValue) continue;

        // ... count up the other squares that have the
        // same set of possibilities
        count = 1;
        for (idx2 = 0; idx2<9; ++idx2) {
            if (idx1 == idx2) continue;
            if (group[idx2]->knownValue) continue;

            if (group[idx1]->possibilities == group[idx2]->possibilities) {
                ++count;
            }
        }

        if (count > 1) {
            Bitmap possibilities = group[idx1]->possibilities;
            int bits = possibilities.count();
            if (bits == count) {
                // this tuple works.  Remove the possibilities from everyone
                // else
                for (int idx3=0; idx3 < 9; ++idx3) {
                    // If it isn't part of the tuple, then remove
                    // the tuple's possibilities from it.
                    if (possibilities != group[idx3]->possibilities) {
                        group[idx3]->possibilities -= possibilities;
                    }
                }
            }
        }
    }
    return NULL;
}


//---------------------------------------------------------------------------
// Pointers algorithm.

void Board::pointerCheck()
{
    Bitmap inHere;      // All possibilities that can go in the three squares
    Bitmap notInHere;   // Possible values in the other squares in the group
    Bitmap cantBeHere;  // Values that can't go in the other 6 squares.
    int i,j;
    int startRow, startCol;
    int row, col;

    //
    // If a value can go in one row of a region, and not in the other rows, then it
    // *must* go in that row of the region.  And since it goes there, it
    // *can't* go in any of the *other* squares in the row.
    //
    // The same is true for columns: if it must go in one column of a region,
    // then it can't go anywhere else in that column
    //
    // TODO: This idea can be extended.  Considering two regions, if
    // there are only two columns that a number can go in (between the
    // two regions), then it must go in those two columns (one from
    // each region) and it MUST go in the third column for the other
    // region.  SudokuOfTheDay.com calls this "double pairs" and
    // "multiple lines"
    //
    // for each region
    for (startRow=0; startRow<9; startRow += 3) {
        for (startCol=0; startCol<9; startCol += 3) {

            // Check each row of the region.
            for (row=startRow; row<startRow+3; ++row) {
                inHere.clear();
                notInHere.clear();

                // Get the values that can go in the row...
                for (j=startCol; j<startCol+3; ++j) {
                    if (!squares[row][j].knownValue) {
                        inHere += squares[row][j].possibilities;
                    }
                }
                // and get the values that can go in the other rows
                for (i=startRow; i<startRow+3; ++i) {
                    if (i==row) continue;
                    for (j=startCol; j<startCol+3; ++j) {
                        notInHere += squares[i][j].possibilities;
                    }
                }

                // Now get the values that can go in the row you're interested
                // in, but can't go in the other rows.
                cantBeHere = inHere - notInHere;

                if (!cantBeHere.isEmpty()) {
                    // [1,2,3] must be in (row,x-y), so they can't be
                    // elsewhere in the row
                    for (j=0; j<9; ++j) {
                        if (j<startCol || j >= startCol+3) {
                            squares[row][j].possibilities -= cantBeHere;
                        }
                    }
                }
            }

            // for each column in the region
            for (col=startCol; col<startCol+3; ++col) {
                inHere.clear();
                notInHere.clear();

                // If it's in this column...
                for (i=startRow; i<startRow+3; ++i) {
                    if (!squares[i][col].knownValue) {
                        inHere += squares[i][col].possibilities;
                    }
                }
                // and it isn't in the other columns...
                for (j=startCol; j<startCol+3; ++j) {
                    if (j==col) continue;
                        for (i=startRow; i<startRow+3; ++i) {
                            notInHere += squares[i][col].possibilities;
                        }
                }

                cantBeHere = inHere - notInHere;

                if (!cantBeHere.isEmpty()) {
                    // Then it can't be elsewhere in the column
                    for (i=0; i<9; ++i) {
                        if (i<startRow || i >= startRow+3) {
                            squares[i][col].possibilities -= cantBeHere;
                        }
                    }
                }
            }
        }
    }

    //
    // If it can go in one region of a row/col and it can't go in the
    // other regions, then it can't be elsewhere in the region
    //

    // for each row
    for (row=0; row<9; ++row) {
        startRow = (row/3) * 3; // start if this row's region

        // for each region within the row
        for (startCol=0; startCol<9; startCol += 3) {
            inHere.clear();
            notInHere.clear();
            // if it's in that region, and not in the other regions....
            for (j=0; j<9; ++j) {
                if (j >= startCol && j <startCol+3) {
                    if (!squares[row][j].knownValue) {
                        inHere += squares[row][j].possibilities;
                    }
                } else {
                    notInHere += squares[row][j].possibilities;
                }
            }

            cantBeHere = inHere - notInHere;

            if (!cantBeHere.isEmpty()) {
                // then it can't be elsewhere within the region
                for (i=startRow; i<startRow+3; ++i) {
                    if (i == row) continue;
                    for (j=startCol; j<startCol+3; ++j) {
                        squares[i][j].possibilities -= cantBeHere;
                    }
                }
            }
        }
    }

    // for each column
    for (col=0; col<9; ++col) {
        startCol = (col/3) * 3; // start if this column's region

        // for each region within the colum
        for (startRow=0; startRow<9; startRow += 3) {
            inHere.clear();
            notInHere.clear();
            // if it's in that region, and not in the other regions....
            for (i=0; i<9; ++i) {
                if (i >= startRow && i <startRow+3) {
                    if (!squares[i][col].knownValue) {
                        inHere += squares[i][col].possibilities;
                    }
                } else {
                    notInHere += squares[i][col].possibilities;
                }
            }

            cantBeHere = inHere - notInHere;

            if (!cantBeHere.isEmpty()) {
                // then it can't be elsewhere within the region
                for (j=startCol; j<startCol+3; ++j) {
                    if (j == col) continue;
                    for (i=startRow; i<startRow+3; ++i) {
                        squares[i][j].possibilities -= cantBeHere;
                    }
                }
            }
        }
    }
}

//---------------------------------------------------------------------------
// Generalized naked Tuples algorithm.

// If N squares contain N values between them, then the N values must
// be distributed amongst those N squares.  This is more general than
// tuples, where each of the N squares must contain exactly the N values.
// For example, if 3 squares contain possibilities {1,2}, {1,3}, and {2,3}, then
// they are a naked tuple for 1,2,3 and thus 1,2,3 can be discounted from all
// other squares in the group.

// The following algorithm is relatively fast:

// Sort squares in the group by number of possibilities (fewest to most)
// Go down the list.  Suppose you have 4 squares with 2 possibilities.
//     - look at all combinations of PAIRS of squares to see if they specify
//       2 possibilities.  This is just the degenerate case: the "tuple" is a pair.
// Now consider all squares with 2, or 3 possibilities:
//     - Look at all combinations of THREE squares.  If they specify 3 possibilities
//       betweem them, then you've found a naked tuple
//         - remove the numbers in the tuple from all OTHER squares in the group
//           I don't think you need to start again now, but you may want to.
// Continue.  Consider all squares with 2,3, or 4 possibilities.
//     - Look at all combinations of FOUR squares.  If they specify 4 possibilties,
//       you've found another one.
// Repeat.

// Remember that there may be no squares with, say, 3 possibilities.


#if 0
// Qsort callback to sort Square *array[] by the number of
// possibilities in each square
static int
arraySort(const void *p1, const void *p2)
{
    Square *s1 = *(Square **)p1;
    Square *s2 = *(Square **)p2;
    return (s1->possibilities.count() - s2->possibilities.count());
}

// ForEachGroup callback to do the naked tuple check.
// Assumes that the board is already reduced.
void *
Board::nakedTupleCheckGroup(Square *inGroup[], void *)
{
    Square *array[9];           // Squares to check for tuple
    unsigned arraySize = 0;     // number of items in array
    Square* group[9];           // The current group
    unsigned last;              // previous number of possibilities
    int i;

    // set array = squares in group with possibilities > 1
    // set "group" to the group you're working on
    for (int idx = 0; idx < 9; ++idx) {
        if (!inGroup[idx]->knownValue) {
            array[arraySize++] = inGroup[idx];
        }
        group[idx] = inGroup[idx];
    }

    // No sense continuing if everything is known.
    if (arraySize == 0) return 0;

    // sort array by number of possibilities in group
    qsort(array, arraySize, sizeof(array[0]), arraySort);

    // Outer loop.  For groups with 2, 3, 4,... possibilities...
    last = array[0]->possibilities.count();
    for (i=0; i < (int)arraySize; ++i) {
        // If you've stepped over a boarder from, say, 3 possibilities to 4...
        if (array[i]->possibilities.count() != last) {
            // Check for tuples in the set of squares.  If you just went
            // from 3 to 6 possibilities, then you check combinations of 3,4,5
            // squares at a time.
            // If you skip from 3 to 6 possibilities, then
            // look for tuples of 3,4, or 5.
            for (; last < array[i]->possibilities.count(); ++last) {
                processCombinations(group, array, i, last);
            }
        }
    }

    // Do the last one
    for (;last <= arraySize; ++last) {
        processCombinations(group, array, i, last);
    }
    return NULL;
}

// Process all combinations of N squares in the array of size arraySize
void
Board::processCombinations(Square *group[], Square *array[],
                           int arraySize, int n)
{
    Square *combo[9];

    if (n > arraySize) {
        return;
    }
    recursiveCombinations(group, array, arraySize, n, combo, 0);
}

void
Board::recursiveCombinations(Square *group[], Square *array[], int arraySize,
                      int n,
                      Square *combo[], int curComboElement)
{
    if (n == 0) {
        nakedTupleCheck(group, combo, curComboElement);
    } else {
        for (int i = 0; i <= arraySize - n; ++i) {
            combo[curComboElement] = array[i];
            recursiveCombinations(group, array+i+1, arraySize-i-1,
                                  n-1,
                                  combo, curComboElement+1);
        }
    }
}

void
Board::nakedTupleCheck(Square *group[], Square *array[], int arraySize)
{
    Bitmap possibilities;
    for (int i = 0; i < arraySize; ++i) {
        possibilities += array[i]->possibilities;
    }
    if (possibilities.count() == (unsigned)arraySize) {
        // You found one!  Remove possibilities from everyone NOT in array.
        for (int j=0; j<9; ++j) {
            bool found = false;
            for (int i=0; i< arraySize; ++i) {
                if (array[i] == group[j]) {
                    found=true;
                    break;
                }
            }
            if (!found) {
                // It isn't in the group.  Re
                group[j]->possibilities &= ~possibilities;
            }
        }
    }
}
#endif

//----------------------------------------------------------
// Hidden Tuples algorithm
//
// Lets start with an example of a hidden tuple.  Suppose you have three
// squares in a group with possibilities (1,2,3), (1,3,4) and (1,2,4).
// Suppose further that possibilities 1,2,3 don't appear anywhere else in the
// group.  In this case, the three squares *must* have the values 1,2,3, although
// you don't know which square has which value.  And since they have 1,2,3, that
// means that any *other* possibilibies, such as 4 in this example, can be
// removed.

// In general, a hidden tuple occurs when N squares in a group contain N possible
// values that don't appear elsewhere in the group.  In that case, the N
// values MUST appear in those N squares, and any OTHER possibilities
// in those squares may be removed.
//
// To check for hidden tuples:
// For all combinations of squares with multiple possibilities:
//  	if  # items unique to this combination == # squares)
//	    	it's a hiddle tuple.  remove everyone else.
//      if # items > #squares then you have a problem!

// How do you tell if items unique to combination?
//	take union of possibilities.
//	remove everyone outside the combination
//  what's left are items that are only within the combination
//	use a bitmap to see who's in and who's out of the combination.

void *
Board::hiddenTupleCallback(Square *inGroup[], void *)
{
    Square *group[9];  // All items in the current group
    Square *array[9];  // Items in the current group with multiple possibilities
    int arraySize=0;   // number of items in array.

    // populate the group and array
    for (int idx = 0; idx < 9; ++idx) {
        group[idx] = inGroup[idx];
        if (!group[idx]->knownValue) {
            array[arraySize++] = group[idx];
        }
    }

    // No sense continuing if it's a trivial case
    if (arraySize < 2) return 0;

    // for each possible tuple size;
    for (int tupleSize = 2; tupleSize <= arraySize; ++tupleSize) {
        // Check all combinations of tuplesize in the array
        Square *combo[9];  // current combination

        recursiveHTCombinations(group, array, arraySize,
                                tupleSize, combo, 0);
    }

    return NULL;
}

// Look for hidden tuples of size tupleSize in the array, which contains
// the items in group that have multiple possibilities.  Array and
// arraySize specify the items to check.  N is the size of the combinations
// to check.  Combo is where we recursively construct each combination of
// elements and curComboElement is the current index into Combo.
void
Board::recursiveHTCombinations(Square *group[],
                                Square *array[],
                                int arraySize,
                                int n,
                                Square *combo[],
                                int curComboElement)
{
    if (n == 0) {
        hiddenTupleCheck(group, combo, curComboElement);
    } else {
        for (int i = 0; i <= arraySize - n; ++i) {
            combo[curComboElement] = array[i];
            recursiveHTCombinations(group, array+i+1, arraySize-i-1,
                                    n-1,
                                    combo, curComboElement+1);
        }
    }
}

void
Board::hiddenTupleCheck(Square *group[], Square *combo[], unsigned comboSize)
{
    Bitmap possibilities;   // the possibilities in the hidden tuple

    // if  # items unique to this combination == # squares)
    //    	it's a hiddle tuple.  remove everyone else.
    // if # items > #squares then you have a problem!

    // Get the union of all possibilities
    // At the same time, mark everyone in the combo.
    for (unsigned i = 0; i < comboSize; ++i) {
        possibilities += combo[i]->possibilities;
        combo[i]->flags += 1;
    }
    //	Now remove the possibilities of everyone outside the combination.
    // What's left is the possibilities that only appear in this combination
    for (unsigned i=0; i < 9; ++i) {
        if (!group[i]->flags.check(1)) {
            possibilities -= group[i]->possibilities;
        }
    }

    // what's left are items that are only within the combination.
    // If the number left equals the size of the combination, then
    // you have a hidden tuple and you can remove all other possibilities
    // from the combination
    if (possibilities.count() == comboSize) {
        for (unsigned i = 0; i < comboSize; ++i) {
            combo[i]->possibilities &= possibilities;
        }
    }

    // Clear the flag in the combo items
    for (unsigned i = 0; i < comboSize; ++i) {
        combo[i]->flags -= 1;
    }
}

#if 0
//----------------------------------------------------------
//   X-Wing algorithm
// Briefly, if there are only two possibilities for a value in 1 row,
// and only two possibilities for the same value in another row, and
// the possibilities are in the same columns, then the 4 possible squares form
// a box and the value must appear in the squares of one diagonal or the other
// diagonal.  In either case, it can't appear in any OTHER spot in the two columns.
//
bool
Board::XWing()
{
    // for each pair of rows
        // get the possibilities that appear exactly twice
        // get the set of columns with unknowns that appear in each row
        // for each pair of columns, check for the value
    int i, j;
    Bitmap atLeastOnce[9];
    Bitmap atLeastTwice[9];
    Bitmap moreThanTwice[9];
    Bitmap exactlyTwice[9];
    Bitmap columnsWithUnknowns[9];
    for (i = 0; i < 9; ++i) {
        for (j=0; j<9; ++j) {
            moreThanTwice[i] += atLeastTwice[i] & squares[i][j].possibilities;
            atLeastTwice[i] += atLeastOnce[i] & squares[i][j].possibilities;
            atLeastOnce[i] += squares[i][j].possibilities;
            if (squares[i][j].knownValue == 0) {
                columnsWithUnknowns[i] += j;
            }
        }
        exactlyTwice[i] = atLeastTwice[i] & ~moreThanTwice[i];
    }

    // Now go back over the rows
    for (i=0 i<8; ++i) {
        if (exactlyTwice[i].isEmpty()) {
            continue;
        }
        // Some values appear exactly twice in this also
        for (int k=i+1; k < 9; ++) {
            if (exactlyTwice[k].isEmpty()) {
                continue;
            }
            Bitmap twiceInBothRows = exactlyTwice[i] & exactlyTwice[k];
            if (twiceInBothRows.isEmpty()) {
                continue;       // No value in common
            }
            Bitmap both = columnsWithUnknowns[i] & columnsWithUnknowns[k];
            if (both.isEmpty()) {
                continue;       // No unknown columns in common
            }
            // Okay, at least one value is in common

            // Get the indices of the columns in common
            int columns[9];
            int numColumns=0;
            for (int x=0; x<9; ++x) {
                if (both.check(x)) {
                    columns[numColumns++] = x;
                }
            }

            // for each pair of columns
            for (int x=0; x < numColumns-1; ++x) {
                for (int y = x+1 y < numColumns; ++y) {
                    int c1 = columns[x];
                    int c2 = columns[y];
                    // If the column and rows all contain a possibility that
                    // appears exactly twice
                    Bitmap theNum = squares[i][c1].possibilities;
                    theNum &= squares[i][c2].possibilities;
                    theNum &= squares[k][c1].possibilities;
                    theNum &= squares[k][c2].possibilities;
                    theNum &= twiceInBothRows;
                    if (!theNum.isEmpty()) {
                        // woo hoo!! Found one
                    }
                }
            }
        }
    }

    return true;
}
#endif

//----------------------------------------------------------
// Solve the board with logic only.  Returns whether it's solved
bool Board::logicSolve()
{
    Board old;
    do {
        // Remember current board
        old = *this;

        // Now go through the algorithms and apply each in turn.  When
        // one doesn't result in any change, try the next one.  In this way
        // we try to solve using the easiest method first, even though we
        // don't currently record the method used.

        // To solve in the smallest amount of code, we could just use three
        // algorithms: pointers, naked tuples and hidden tuples.  The easier
        // algorithms (reduce, scan, pairs and tuples) are just degenerate cases
        // of naked and hidden tuples.

        // reduce
        reduce();
        setKnownValues();
        if (old != *this) continue;

        // scan
        for (int val=1; val<10; ++val) {
            forEachGroup(&Board::scan, (void *)val);
        }
        reduce();
        setKnownValues();
        if (old != *this) continue;

        //pairs
        forEachGroup(&Board::pairCheck, NULL);
        setKnownValues();
        if (old != *this) continue;

        // tuples
        forEachGroup(&Board::tupleCheck, NULL);
        setKnownValues();
        if (old != *this) continue;

        // Pointers
        pointerCheck();
        setKnownValues();
        if (old != *this) continue;

	// Skip the naked tuple check because it requires qsort,
	// which isn't implemented
#if 0
        // Naked Tuples
        forEachGroup(&Board::nakedTupleCheckGroup, NULL);
        setKnownValues();
        if (old != *this) continue;
#endif

        // Hidden Tuples
        forEachGroup(&Board::hiddenTupleCallback, NULL);
        setKnownValues();
        if (old != *this) continue;
    } while (*this != old && !isSolved());
    return isSolved() && isValid();
}

// Attempt to solve with guessing allowed
bool
Board::solve()
{
    // First try solving with logic alone.
    if (logicSolve()) {
        return true;
    }

    // Now you have to guess.

    // Find a square with the minimum number of possibilities and
    // try them all.

    // Find square with minimum possibilities
    Square *s = 0;              // Points to square with min possibilities
    unsigned minPossible = INT_MAX;  // The minimum number of possibilities
    unsigned  i,j;

    for (i=0; i < 9; ++i) {
        for (j=0; j<9; ++j) {
            unsigned c = squares[i][j].possibilities.count();
            if (c > 1 && c < minPossible) {
                minPossible = c;
                s = &squares[i][j];
            }
        }
    }

    if (s == 0) {
        // Must have all values set but it wasn't solved according to
        // call to logicSolve above.  This can happen if you fill in all values
        // and get an invalid board. So return false.
        return false;
    }

    Board oldBoard(*this);  // remember the old board;
    Bitmap possibility(s->possibilities);   // remember possibilities of this square

    // Now for each of the possibible values of s, assign the value to s and try
    // to solve recursively
    for (i=1; i <= 9; ++i) {
        if (possibility.check(i)) {
            setAndReduce(*s, i);
            reduce();
            if (solve()) {
                return true;
            }
            // Otherwise restore board and move on to the next possibility
            *this = oldBoard;
        }
    }
    // If you get to here and you didn't solve it then restore the old
    // board and return.  You need to restore the old board because you
    // may have been called recursively with a guess that turned out to be wrong.
    *this = oldBoard;
    return false;
}


// Set a board from an array of characters
bool
Board::fromString(const char *str)
{
    int count = 0;
    clear();
    for (int i = 0; i < 9; ++i) {
	for (int j=0; j < 9; ++j,++count) {
	    if (str[count] <= '9' && str[count] >= '1') {
		squares[i][j].setVal(str[count] - '0');
	    }
	}
    }
    return true;
}

bool
Board::toString(char *str)
{
    int count = 0;
    for (int i = 0; i < 9; ++i) {
	for (int j=0; j < 9; ++j) {
	    int kv;
	    kv = squares[i][j].knownValue;
	    // kv = squares[i][j].possibilities.count();
	    str[count++] = kv + '0';
	}
    }
    str[count] = 0;		// null terminate
    return true;
}

