//---------------------------------------------------------------------------
#ifndef BoardH
#define BoardH
#include "Square.h"

//---------------------------------------------------------------------------
#endif

// This class implements a sudoku board

class Board {
public:
    Board() :
        next(NULL)
        { clear(); }
    ~Board();
    Board(const Board &b)   // Copy constructor
        {*this = b; }
    Board &operator=(const Board &b);          // assignment operator

    bool operator == (const Board &b) const;
    bool operator != (const Board &b) const
        { return !(*this == b); }

    // Clear all values
    void clear();

    // IsValid() returns true if the board is valid (no two known values are the
    // same in any row, column, or region).  validateGroup is a forEachGroup()
    // helper function to validate one group.
    bool isValid();
    void *validateGroup(Square *group[], void *);

    // Reduce() looks at the known values in each group and removes them from
    // the set of possibilities in the unknown values.  reduceDB() is a forEachGroup()
    // callback function that reduces a single group
    void reduce();
    void *reduceCB(Square *group[], void *param);

    // Set the square to the known value val and then remove val from
    // the possibilities of all squares in the same row, column and region.
    void setAndReduce(Square &s, int val);

    // Thow away the possible values and call reduce.  In other words, set
    // the possible values to something valid.  Note that this effectively removes
    // any information gleened from the Pointers algorithym and others.
    void calcPossibilities();

    // Look at the possibilities in the squares and set known values for
    // for any unknown values that have just one possibility;
    void setKnownValues();

    // Given the x&y coordinates for a square with a known value, remove
    // that value from the possibilities of the row, column, and region that
    // the square belongs to
    void reduceSingle(int i, int j);

    // forEachGroup lets you call a Board method that processes each group in the
    // puzzle (each row, column, and region).  The indices of the squares are
    // passed to FUNC() in IVALS and JVALS. FUNC also gets passed the PARAMS
    // that you pass in.  FUNC should return NULL unless forEachGroup should be
    // aborted, in which case forEachGroup will return the first non-zero
    // return value from FUNC.
    typedef void *(Board::*ForEachGroupCallback)(Square *group[], void *params);
    void *forEachGroup(ForEachGroupCallback func, void *params);

    // Scan Algorithm.  AKA "single choice for number"
    // This is a Callback to forEachGroup.
    // This method checks to see if there is only one possible place for
    // the value v in the group and sets it if possible.
    void *scan(Square *group[], void *v);

    // ForEachGroup callback to check for pairs/tuples.  For example, if there are
    // two squares in the group whose only possibilities are 4 and 5, then one
    // of them MUST be 4 and the other MUST be 5.  Therefore, no OTHER squares
    // in the group can be either 4 or 5.  The same is true for larger tuples.
    void *pairCheck(Square *group[], void *);
    void *tupleCheck(Square *group[], void *);

    // If all possibilities for a value within a region
    // lie in one row, then the other squares within that
    // ROW cannot contain the value
    void pointerCheck();

    // stuff for naked tuples algorithm
    void *nakedTupleCheckGroup(Square *group[], void *);
    void processCombinations(Square *group[], Square *array[], int arraySize, int n);
    void recursiveCombinations(Square *group[], Square *array[], int arraySize,
                      int n,
                      Square *combo[], int curComboElement);
    void nakedTupleCheck(Square *group[], Square *array[], int arraySize);


    // Methods for hidden tuples algorithm
    void *hiddenTupleCallback(Square *group[], void *);
    void processHTCombinations(Square *group[], Square *array[], int tupleSize);
    void recursiveHTCombinations(Square *group[],
                                Square *array[], int arraySize,
                                int n,
                                Square *combo[], int curComboElement);
    void hiddenTupleCheck(Square *group[], Square *combo[], unsigned comboSize);

    // XWing algorithm
    bool xWing();
    
    bool logicSolve();  // attempt to solve with logic only. Return whether it's solved.
    bool solve();       // attempt to solve with guessing allowed.

    // Given a pointer to a square, return its coordinates. The square must
    // be one in the "squares" array below
    void coordinates(int &i, int &j, const Square &s) const;

    // I/O routines
    bool read(FILE *fp);
    bool write(FILE *fp);


    // For hpgcc version.  These copy to/from an 82 char string.
    // Empty cells are represented by zero
    bool toString(char *str);
    bool fromString(const char *str);

    // return true of the board is solved.
    bool isSolved();

    Square squares[9][9];
    Board *next;    // next in a list
private:
};

