//---------------------------------------------------------------------------
#ifndef bitmapH
#define bitmapH
//---------------------------------------------------------------------------

// This implements a set of numbers 0-31.  It's implemented as a 32 bit bitmap
// and is very fast.
class Bitmap {
public:
    Bitmap();

    // Add or subtract a single value from the bitmap
    Bitmap operator+(int val) const;
    Bitmap &operator +=(int val);
    Bitmap operator-(int val) const;
    Bitmap &operator -=(int val);

    // Add or subtract one bitmap from another.  When subtracting, the
    // subtracting bitmap doesn't need to be a subset.
    Bitmap operator+(const Bitmap) const;
    Bitmap &operator+=(const Bitmap);
    Bitmap operator-(const Bitmap) const;
    Bitmap &operator-=(const Bitmap);

    // Intersection
    Bitmap operator&(const Bitmap) const;
    Bitmap &operator&=(const Bitmap);

    bool operator ==(const Bitmap &) const;
    bool operator !=(const Bitmap &) const;

    // Subset
    bool operator <(const Bitmap &) const;

    // Superset
    bool operator >(const Bitmap &) const;

    // Invert a bitmap.
    Bitmap operator~() const;

    // Number of members
    unsigned count() const;
    bool isEmpty() const;
    bool check(int v) const;
    void clear();

private:
    unsigned long val;      // must be 32 bits
};



//-------------------------------------------------------------
//
// Implementation


inline Bitmap::Bitmap() : val(0) {;}

// Add or subtract a single value from the bitmap
inline Bitmap Bitmap::operator+(int v) const
{
    Bitmap result(*this);
    return result += val;
}

inline Bitmap &Bitmap::operator +=(int v)
{
    if (v >= 0 && v < 32) {
        val |= (1 << v);
    }
    return *this;
}

inline Bitmap Bitmap::operator-(int v) const
{
    Bitmap result(*this);
    return result -= v;
}

inline Bitmap &Bitmap::operator -=(int v)
{
    if (v >= 0 && v < 32) {
        val &= ~(1 << v);
    }
    return *this;
}


// Add or subtract one bitmap from another.  When subtracting, the
// subtracting bitmap doesn't need to be a subset.
inline Bitmap Bitmap::operator+(const Bitmap src) const
{
    Bitmap result(*this);
    result += src;
    return result;
}

inline Bitmap &Bitmap::operator+=(const Bitmap src)
{
    val |= src.val;
    return *this;
}

inline Bitmap Bitmap::operator-(const Bitmap src) const
{
    Bitmap result(*this);
    result -= src;
    return result;
}

inline Bitmap &Bitmap::operator-=(const Bitmap src)
{
    val &= ~src.val;
    return *this;
}

// Intersection
inline Bitmap Bitmap::operator&(const Bitmap src) const
{
    Bitmap result(*this);
    result &= src;
    return result;
}

inline Bitmap &Bitmap::operator&=(const Bitmap src)
{
    val &= src.val;
    return *this;
}


inline bool Bitmap::operator ==(const Bitmap &src) const
{
    return val == src.val;
}
inline bool Bitmap::operator !=(const Bitmap &src) const
{
    return val != src.val;
}

// lhs < rhs if lhs is a subset of rhs
inline bool Bitmap::operator <(const Bitmap &src) const
{
    // True if the union is the rhs
    return (val | src.val) == src.val;
}


// Superset
inline bool Bitmap::operator >(const Bitmap &src) const
{
    return (val | src.val) == val;
}

// Invert a bitmap.
inline Bitmap Bitmap::operator~() const
{
    Bitmap result;
    result.val = ~val;
    return result;
}

inline bool Bitmap::check(int v) const
{
    return (v >= 0 && v < 32 && (val & (1 << v)));
}

inline void Bitmap::clear()
{
    val = 0;
}

inline bool Bitmap::isEmpty() const
{
    return val == 0;
}

#endif
