#include <hpgcc49.h>
#include "hpobjects.h"


extern int solve_sudoku(char *str);
char *matrix2String(SatAddr m);
SatAddr string2Matrix(const char *str);

char errString[100];		/* error string */
// Try to solve a sudolu puzzle.
// Inputs:
//      Level 1: - an 81 character string representing the puzzle, or
//               - a 9x9 symbolic matrix of reals or zints representing
//                 the puzzle, or
//               - a 9x9 array of real or ZINT representing the puzzle
// Outputs one of the following:
//      Level 1 contains 0. and Level 2 contains an error message
//      Level 1 contains 1. and Level 2 contains solved board as a
//              9x9 matrix of zint's
//      Level 1 contains 2. and Level 2 contains a partially solved board
//              as a 9x9 matrix of zints
int
main(void)
{
    SatAddr arg;
    char *str = 0;		/* 81 char string */
    int depth;
    int result;			/* result status to push */
    bool argsOk = false;	/* argument is okay */
    clear_screen();
    
    // Get stack level 1 and convert to an 81 character string
    depth = STACKdepth();
    arg = STACKpop();
    if (isSTRING(arg)) {
	str = STRINGdecode(arg, 0);
	argsOk = (str != NULL);
    } else if (isMATRIX(arg)) {
	str = matrix2String(arg);
	argsOk = (str != NULL);
    } else if (depth == 0) {
	strcpy(errString, "Need a 9x9 array");
    } else {
	strcpy(errString, "Wrong type. Need a 9x9 array");
    }
	
    if (argsOk) {
	// Make sure the length is right
	if (strlen(str) != 81) {
	    strcpy(errString, "Wrong size. Need a 9x9 array");
	    argsOk = false;
	}
    }

    // Return error if there was a problem. At this point,
    // errString contains the error string
    if (!argsOk) {
	STACKpush(STRINGencode(errString, 0));
	STACKpush(REALencode(0.0, 0));
	return 0;
    }

    sys_slowOff();		// crank up the clock
    result = 0;			/* assume you'll fail */
    switch(solve_sudoku(str)) {
    case 1:
	if ((result = string2Matrix(str))) {
	    STACKpush(result);
	    result = 1;
	} else {
	    STACKpush(STRINGencode("Can't encode result", 0));
	}
	break;
    case 0:
	// failed. str contains error desription
	STACKpush(STRINGencode(str,0));
	break;
    case 2:
	// partial solution
	STACKpush(string2Matrix(str));
	STACKpush(STRINGencode("Can't solve completely",0));
    }

    // push the result
    STACKpush(REALencode(result, 0));

    sys_slowOn();		// slow the clock again
    return 0;			/* done */
}

// Convert the matrix to a string.  Returns the string on success,
// or on failure redturns NULL and sets errString
char *matrix2String(SatAddr m)
{
    SatAddr iter, iter2;
    SatAddr subMatrix;		/* one of the submatrices */
    SatAddr obj;		/* object within submatrix */
    int i;			/* value in submatrix */
    long long ll;
    double d;

    char str[82];
    char *cp = str;

    if (MATRIXsize(m) != 9) {
	sprintf(errString, "Wrong size. Need a 9x9 array");
	return NULL;		/* wrong size */
    }

    // For each submatrix
    for (subMatrix = MATRIXfirstOb(m, &iter);
	 subMatrix;
	 subMatrix = MATRIXnextOb(&iter)) {

	if (MATRIXsize(subMatrix) != 9) {
	    // not a matrix or size is bad.
	    sprintf(errString, "Wrong size. Need a 9x9 array");
	    return NULL;
	}

	// The submatrix should contain reals or zints
	for (obj = MATRIXfirstOb(subMatrix, &iter2);
	     obj;
	     obj = MATRIXnextOb(&iter2)) {

	    if (isREAL(obj)) {
		REALdecode(obj, &d);
		i = (int) d;
		if (d != (double) i) {
		    sprintf(errString, "Bad value %g in input array\n", d);
		    return NULL;
		}
	    } else if (isZINT(obj)) {
		ZINTdecodell(obj, &ll);
		i = ll;
	    } else {
		// wrong type
		sprintf(errString, "Wrong type. Need a 9x9 array");
		return NULL;
	    }
	    if (i < 0 || i > 9) {
		sprintf(errString, "Bad value %d in input array", i);
		return NULL;
	    } else {
		*cp++ = '0'+i;
	    }
	}
    }
    *cp = 0;
    return strdup(str);
}

SatAddr
string2Matrix(const char *str)
{
    SatAddr zints[9];
    SatAddr subs[9];
    SatAddr result;
    int i,j;
    long long val;
    const char *cp = str;

    for (i=0; i<9; ++i) {
	for (j=0; j<9; ++j) {
	    val = *cp++ - '0';
	    zints[j] = ZINTencodell(val, 0);
	    if (zints[j] == 0) {
		return 0;
	    }
	}
	subs[i] = MATRIXencodeN(9, zints, 0);
	if (subs[i] == 0) {
	    return 0;
	}
    }
    result = MATRIXencodeN(9, subs, 0);
    return result;
}
